//****************************************************************************
// TPM2NET TO WS2801 Soft SPI
// Arduino Uno code by neophob // http://neophob.com/
//
// For Netino with Atmega644p 
// http://code.google.com/p/avr-netino/
// 
// ENC28J60 Ethercard Library
// https://github.com/jcw/ethercard
//
// Configured for Netino(644P) with ENC28J60 by leddie / 04.01.2013 
//*****************************************************************************

#include <SPI.h>  // For WS2801 declarations
#include <EtherCard.h>  // enc28j60 lib
#include <WS2801.h> // SPI Pins conflicts with enc28j60 -> Using WS2801 Soft SPI instead

// tpm2net constants
#define TPM2NET_HEADER_SIZE 			5
#define TPM2NET_HEADER_IDENT 			0x9c
#define TPM2NET_CMD_DATAFRAME 			0xDA
#define TPM2NET_FOOTER_IDENT 			0x36
#define NR_OF_PANELS 					1
#define PIXELS_PER_PANEL 				300
#define UDP_PACKET_SIZE 				900

static unsigned int NETIO_CSPIN_ENC28J60 = 28; // ENC28J60 CSPIN

byte macMy[]         = {0x00, 0x22, 0xF9, 0x01, 0x30, 0xDA };  // Netino Mac
byte ipMy[]          = {192, 168, 100, 100};                     // Netino IP
word portMy          = 65506;                                   // TPM2NET listening port


// buffer for receiving and sending data
uint8_t Ethernet::buffer[UDP_PACKET_SIZE];  


// Set the 2 pins you will use for SOFT SPI Ouput
int dataPin = 8;  // pin40  on mega644p
int clockPin = 9; // pin39  on mega644p

//initialize pixels with soft spi pins
WS2801 strip = WS2801(PIXELS_PER_PANEL*NR_OF_PANELS, dataPin, clockPin);


void setup () {
  
  ether.begin(sizeof Ethernet::buffer, macMy, NETIO_CSPIN_ENC28J60);
  ether.hisport = portMy;  
  ether.staticSetup(ipMy);
       
  memset(Ethernet::buffer, 0, UDP_PACKET_SIZE); 
   
  strip.begin();  
   
}
 
 void loop () {

  word len = ether.packetReceive();  // receive new packets
  word pos = ether.packetLoop(len);  // respond to incoming pings
  
  byte *ptr = &ether.buffer[pos];

  if (len>TPM2NET_HEADER_SIZE) {
    
  // -- Header check
    if (ptr[0]!=TPM2NET_HEADER_IDENT) {
      return;
    }
    
  // -- Data Frame check
    if (ptr[1]!=TPM2NET_CMD_DATAFRAME) {
      return;
    }
  

  uint16_t frameSize = ((ptr[3] << 0) & 0xFF) + ((ptr[2] << 8) & 0xFF00); // is this correct?

  uint8_t packetNumber = ptr[4];

  uint16_t currentLed = packetNumber*PIXELS_PER_PANEL;
 
  int x=TPM2NET_HEADER_SIZE;
  for (byte i=0; i < frameSize; i++) {
   strip.setPixelColor(currentLed++, ptr[x], ptr[x+1], ptr[x+2]);
     x+=3;
    }
   
   strip.show();   // write all the pixels out  

  //check footer
    frameSize += TPM2NET_HEADER_SIZE;
    if (ptr[frameSize]!=TPM2NET_FOOTER_IDENT) {
      return;
    }
   }
 }

